import { tryRequire } from "dr-js/module/env/tryRequire";
import { catchAsync } from "dr-js/module/common/error";
import { isBasicFunction } from "dr-js/module/common/check";
const DENY_ALL = () => !1;
const ALLOW_ALL = () => !0;
const configurePermission = async ({permissionType, permissionFunc, permissionFile, logger}) => {
  let configurePermissionFunc;
  switch (permissionType) {
   case "deny":
    logger.add("use permission: deny-all");
    return {
      checkPermission: DENY_ALL
    };
   case "allow":
    logger.add("use permission: allow-all");
    return {
      checkPermission: ALLOW_ALL
    };
   case "func":
    if (!isBasicFunction(configurePermissionFunc = permissionFunc)) throw new Error("invalid permissionFunc");
    logger.add("use permissionFunc");
    break;
   case "file":
    configurePermissionFunc = (tryRequire(permissionFile) || {
      configurePermission: null
    }).configurePermission;
    if (!isBasicFunction(configurePermissionFunc)) throw new Error(`failed to load permissionFile: ${permissionFile}`);
    logger.add("use permissionFile");
    break;
   default:
    throw new Error(`invalid permissionType: ${permissionType}`);
  }
  const {result, error} = await catchAsync(configurePermissionFunc, {
    logger
  });
  if (error) throw error;
  if (!isBasicFunction(result.checkPermission)) throw new Error("expect generated checkPermission function");
  return result;
};
export { configurePermission };