import { createSecureContext } from "tls";
import { createServer } from "dr-js/module/node/server/Server";
import { readFileAsync } from "dr-js/module/node/file/function";
const loadFile = filePath => filePath ? readFileAsync(filePath) : null;
const configureServer = async ({protocol = "http:", hostname = "localhost", port, fileSSLKey, fileSSLCert, fileSSLChain, fileSSLDHParam, ...extraOption}) => {
  const isHttps = "https:" === protocol;
  return createServer({
    protocol,
    hostname,
    port,
    key: await loadFile(isHttps && fileSSLKey),
    cert: await loadFile(isHttps && fileSSLCert),
    ca: await loadFile(isHttps && fileSSLChain),
    dhparam: await loadFile(isHttps && fileSSLDHParam),
    ...extraOption
  });
};
const getServerSNIOption = async hostnameConfigMap => {
  const secureContextMap = {};
  for (const [hostname, {fileSSLKey, fileSSLCert, fileSSLChain}] of Object.entries(hostnameConfigMap)) secureContextMap[hostname] = {
    key: await loadFile(fileSSLKey),
    cert: await loadFile(fileSSLCert),
    ca: await loadFile(fileSSLChain)
  };
  const defaultSecureContext = Object.keys(secureContextMap)[0];
  return {
    ...defaultSecureContext,
    SNICallback: (hostname, callback) => callback(null, createSecureContext(secureContextMap[hostname] || defaultSecureContext))
  };
};
export { configureServer, getServerSNIOption };