const pathContentStyle = "<style>\nh2, h6 { margin: 0.5em 4px; }\n.select, .directory, .file { display: flex; flex-flow: row nowrap; align-items: stretch; }\n.directory:hover, .file:hover { background: #eee; }\n.name { overflow:hidden; flex: 1; white-space:nowrap; text-align: left; text-overflow: ellipsis; background: transparent; }\n.select .name, .file .name { pointer-events: none; color: #666; }\n.edit { pointer-events: auto; min-width: 1.5em; min-height: auto; line-height: normal; }\n</style>";
const initPathContent = (URL_PATH_ACTION, URL_FILE_SERVE, IS_READ_ONLY = !0, PATH_ACTION_TYPE, authFetch, withConfirmModal, withPromptModal) => {
  const {open, qS, cE, aCL, Dr: {Common: {Format, Compare: {compareStringWithNumber}}}} = window;
  const SORT_FUNC = {
    NAME: ([nameA], [nameB]) => compareStringWithNumber(nameA, nameB),
    TIME: ([, , mtimeMsA = 0], [, , mtimeMsB = 0]) => mtimeMsB - mtimeMsA,
    SIZE: ([, sizeA], [, sizeB]) => sizeB - sizeA
  };
  const SORT_TYPE_LIST = Object.keys(SORT_FUNC);
  const pathPush = (relativePath, name) => "." === relativePath ? name : `${relativePath}/${name}`;
  const pathName = relativePath => "." === relativePath ? "[ROOT]" : `${relativePath}/`;
  const initialPathContentState = {
    pathSortType: SORT_TYPE_LIST[0],
    pathContent: {
      relativePath: ".",
      directoryList: [],
      fileList: []
    }
  };
  const authFetchPathAction = async bodyObject => (await authFetch(URL_PATH_ACTION, {
    method: "POST",
    body: JSON.stringify(bodyObject)
  })).json();
  const doLoadPath = async (pathContentStore, relativePath = pathContentStore.getState().pathContent.relativePath) => {
    const {resultList: [{key: nextRelativePath, directoryList, fileList}]} = await authFetchPathAction({
      nameList: [ "" ],
      actionType: PATH_ACTION_TYPE.DIRECTORY_CONTENT,
      key: relativePath || "."
    });
    pathContentStore.setState({
      pathContent: {
        relativePath: nextRelativePath || ".",
        directoryList,
        fileList
      }
    });
  };
  return {
    initialPathContentState,
    cyclePathSortType: (pathContentStore, pathSortType = pathContentStore.getState().pathSortType) => {
      const nextSortIndex = (SORT_TYPE_LIST.indexOf(pathSortType) + 1) % SORT_TYPE_LIST.length;
      pathContentStore.setState({
        pathSortType: SORT_TYPE_LIST[nextSortIndex]
      });
    },
    getLoadPathAsync: pathContentStore => async relativePath => doLoadPath(pathContentStore, relativePath),
    getPathActionAsync: pathContentStore => async (nameList, actionType, key, keyTo) => {
      if (actionType !== PATH_ACTION_TYPE.PATH_MOVE && actionType !== PATH_ACTION_TYPE.PATH_COPY || keyTo && keyTo !== key) {
        await authFetchPathAction({
          nameList,
          actionType,
          key,
          keyTo
        });
        await doLoadPath(pathContentStore);
      }
    },
    getPreviewFile: (pathContentStore, authUrl) => async (relativePath, fileName) => open(authUrl(`${URL_FILE_SERVE}/${encodeURIComponent(pathPush(relativePath, fileName))}`)),
    getDownloadFile: (pathContentStore, authDownload) => async (relativePath, fileName) => authDownload(`${URL_FILE_SERVE}/${encodeURIComponent(pathPush(relativePath, fileName))}`, fileName),
    renderPathContent: (pathContentStore, parentElement, loadPath, pathAction, previewFile, downloadFile) => {
      const {pathSortType, pathContent: {relativePath, directoryList, fileList}} = pathContentStore.getState();
      const selectToggleMap = {};
      const selectNameSet = new Set();
      const doSelectRemaining = () => {
        Object.entries(selectToggleMap).forEach(([name, toggle]) => !selectNameSet.has(name) && toggle());
        updateSelectStatus();
      };
      const TEXT_BATCH = text => `🗃️${text}`;
      const selectEditSelectNone = cE("button", {
        className: "edit",
        innerText: TEXT_BATCH("☐"),
        onclick: doSelectRemaining
      });
      const selectEditSelectSome = cE("button", {
        className: "edit",
        innerText: TEXT_BATCH("☒"),
        onclick: doSelectRemaining
      });
      const selectEditSelectAll = cE("button", {
        className: "edit",
        innerText: TEXT_BATCH("☑"),
        onclick: () => {
          selectNameSet.forEach(name => selectToggleMap[name]());
          updateSelectStatus();
        }
      });
      const selectEditMove = cE("button", {
        className: "edit",
        innerText: TEXT_BATCH("✂️"),
        onclick: async () => pathAction([ ...selectNameSet ], PATH_ACTION_TYPE.PATH_MOVE, relativePath, await withPromptModal(`Batch Move ${selectNameSet.size} Path To`, relativePath))
      });
      const selectEditCopy = cE("button", {
        className: "edit",
        innerText: TEXT_BATCH("📋"),
        onclick: async () => pathAction([ ...selectNameSet ], PATH_ACTION_TYPE.PATH_COPY, relativePath, await withPromptModal(`Batch Copy ${selectNameSet.size} Path To`, relativePath))
      });
      const selectEditDelete = cE("button", {
        className: "edit",
        innerText: TEXT_BATCH("🗑️"),
        onclick: async () => await withConfirmModal(`Batch Delete ${selectNameSet.size} Path In: ${pathName(relativePath)}?`) && pathAction([ ...selectNameSet ], PATH_ACTION_TYPE.PATH_DELETE, relativePath)
      });
      const updateSelectStatus = () => aCL(qS(".select", ""), [ selectNameSet.size ? selectNameSet.size < directoryList.length + fileList.length ? selectEditSelectSome : selectEditSelectAll : selectEditSelectNone, selectNameSet.size && selectEditMove, selectNameSet.size && selectEditCopy, selectNameSet.size && selectEditDelete, cE("span", {
        className: "name button",
        innerText: `${selectNameSet.size} selected`
      }) ]);
      const renderSelectButton = name => {
        selectToggleMap[name] = () => {
          const prevIsSelect = selectNameSet.has(name);
          selectNameSet[prevIsSelect ? "delete" : "add"](name);
          const isSelect = !prevIsSelect;
          element.className = isSelect ? "edit select" : "edit";
          element.innerText = isSelect ? "☑" : "☐";
        };
        const element = cE("button", {
          className: "edit",
          innerText: "☐",
          onclick: () => {
            selectToggleMap[name]();
            updateSelectStatus();
          }
        });
        return element;
      };
      const renderCommonEditList = relativePath => IS_READ_ONLY || window.innerWidth <= 480 ? [] : [ cE("button", {
        className: "edit",
        innerText: "✂️",
        onclick: async () => pathAction([ "" ], PATH_ACTION_TYPE.PATH_MOVE, relativePath, await withPromptModal("Move To", relativePath))
      }), cE("button", {
        className: "edit",
        innerText: "📋",
        onclick: async () => pathAction([ "" ], PATH_ACTION_TYPE.PATH_COPY, relativePath, await withPromptModal("Copy To", relativePath))
      }), cE("button", {
        className: "edit",
        innerText: "🗑️",
        onclick: async () => await withConfirmModal(`Delete path: ${relativePath}?`) && pathAction([ "" ], PATH_ACTION_TYPE.PATH_DELETE, relativePath)
      }) ];
      parentElement.innerHTML = "";
      aCL(parentElement, [ cE("h2", {
        innerText: pathName(relativePath)
      }), cE("h6", {
        innerText: `${directoryList.length} directory, ${fileList.length} file (${Format.binary(fileList.reduce((o, [, size]) => o + size, 0))}B)`
      }), "." !== relativePath && cE("div", {
        className: "directory"
      }, [ cE("button", {
        className: "name",
        innerText: "🔙|..",
        onclick: () => loadPath((relativePath => "." === relativePath ? "." : relativePath.split("/").slice(0, -1).join("/"))(relativePath))
      }) ]), !IS_READ_ONLY && cE("div", {
        className: "select"
      }), ...directoryList.sort((nameA, nameB) => SORT_FUNC[pathSortType]([ nameA ], [ nameB ])).map(name => cE("div", {
        className: "directory"
      }, [ !IS_READ_ONLY && renderSelectButton(name), cE("button", {
        className: "name",
        innerText: `📁|${name}/`,
        onclick: () => loadPath(pathPush(relativePath, name))
      }), ...renderCommonEditList(pathPush(relativePath, name)) ])), ...fileList.sort(SORT_FUNC[pathSortType]).map(([name, size, mtimeMs]) => cE("div", {
        className: "file"
      }, [ !IS_READ_ONLY && renderSelectButton(name), cE("span", {
        className: "name button",
        innerText: `📄|${name} - ${new Date(mtimeMs).toLocaleString()}`
      }), cE("button", {
        className: "edit",
        innerText: `${Format.binary(size)}B|💾`,
        onclick: () => downloadFile(relativePath, name)
      }), cE("button", {
        className: "edit",
        innerText: "🔍",
        onclick: () => previewFile(relativePath, name)
      }), ...renderCommonEditList(pathPush(relativePath, name)) ])) ]);
      !IS_READ_ONLY && updateSelectStatus();
    }
  };
};
export { pathContentStyle, initPathContent };