import { BASIC_EXTENSION_MAP } from "dr-js/module/common/module/MIME";
import { responderSendBufferCompress, prepareBufferDataAsync } from "dr-js/module/node/server/Responder/Send";
import { getRouteParamAny } from "dr-js/module/node/server/Responder/Router";
import { AUTH_SKIP } from "../Auth/configure";
import { PATH_ACTION_TYPE } from "./task/pathAction";
import { getHTML } from "./HTML/main";
import { createResponderPathAction, createResponderServeFile, createResponderFileChunkUpload, createResponderStorageStatus } from "./responder";
import { PERMISSION_EXPLORER_PATH_ACTION, PERMISSION_EXPLORER_FILE_UPLOAD_START } from "./permission";
const configureFeaturePack = async ({option, logger, routePrefix = "", featureAuth: {authPack: {authMode}, createResponderCheckAuth, URL_AUTH_CHECK}, explorerRootPath: rootPath, explorerUploadMergePath: mergePath, checkPermission = (type, payload) => !0}) => {
  const URL_HTML = `${routePrefix}/explorer`;
  const URL_PATH_ACTION = `${routePrefix}/path-action`;
  const URL_FILE_SERVE = `${routePrefix}/file-serve`;
  const URL_FILE_UPLOAD = `${routePrefix}/file-chunk-upload`;
  const URL_STORAGE_STATUS = `${routePrefix}/storage-status`;
  const IS_SKIP_AUTH = authMode === AUTH_SKIP;
  const IS_READ_ONLY = !mergePath;
  const HTMLBufferData = await prepareBufferDataAsync(Buffer.from(getHTML({
    URL_AUTH_CHECK,
    URL_PATH_ACTION,
    URL_FILE_SERVE,
    URL_FILE_UPLOAD,
    URL_STORAGE_STATUS,
    IS_SKIP_AUTH,
    IS_READ_ONLY,
    PATH_ACTION_TYPE
  })), BASIC_EXTENSION_MAP.html);
  const responderPathAction = createResponderPathAction({
    rootPath,
    logger
  });
  const responderFileServe = createResponderServeFile({
    rootPath
  });
  const responderFileChunkUpload = IS_READ_ONLY ? store => {} : await createResponderFileChunkUpload({
    rootPath,
    mergePath,
    logger
  });
  const responderStorageStatus = createResponderStorageStatus({
    rootPath
  });
  const routeList = [ [ URL_HTML, "GET", store => responderSendBufferCompress(store, HTMLBufferData) ], [ URL_PATH_ACTION, "POST", createResponderCheckAuth({
    responderNext: async store => {
      const {nameList, actionType, key, keyTo} = await store.requestJSON();
      if (IS_SKIP_AUTH || await checkPermission(PERMISSION_EXPLORER_PATH_ACTION, {
        store,
        actionType
      })) return responderPathAction(store, nameList, actionType, key, keyTo);
    }
  }) ], [ `${URL_FILE_SERVE}/*`, "GET", createResponderCheckAuth({
    responderNext: store => responderFileServe(store, decodeURIComponent(getRouteParamAny(store)))
  }) ], [ URL_FILE_UPLOAD, "POST", createResponderCheckAuth({
    responderNext: store => responderFileChunkUpload(store, {
      onUploadStart: async ({filePath, key}) => {
        if (!IS_SKIP_AUTH && !await checkPermission(PERMISSION_EXPLORER_FILE_UPLOAD_START, {
          store,
          filePath,
          key,
          IS_READ_ONLY
        })) throw new Error(`deny file upload: ${key}`);
      }
    })
  }) ], [ URL_STORAGE_STATUS, "GET", createResponderCheckAuth({
    responderNext: responderStorageStatus
  }) ] ].filter(Boolean);
  return {
    URL_HTML,
    URL_PATH_ACTION,
    URL_FILE_SERVE,
    URL_FILE_UPLOAD,
    URL_STORAGE_STATUS,
    routeList
  };
};
export { configureFeaturePack };