import { dirname } from "path";
import { withRetryAsync } from "dr-js/module/common/function";
import { percent, binary } from "dr-js/module/common/format";
import { readFileAsync, writeFileAsync } from "dr-js/module/node/file/function";
import { createDirectory } from "dr-js/module/node/file/File";
import { uploadFileByChunk } from "./fileChunkUpload";
const fileUpload = async ({fileInputPath, fileBuffer, key, urlFileUpload, timeout = 3e4, maxRetry = 3, wait = 1e3, authFetch, logger, log = logger && logger.add}) => {
  void 0 === fileBuffer && (fileBuffer = await readFileAsync(fileInputPath));
  log && log(`[Upload] key: ${key}, size: ${binary(fileBuffer.length)}B`);
  await uploadFileByChunk({
    fileBuffer,
    key,
    onProgress: (uploadedSize, totalSize) => log && log(`[Upload] upload: ${percent(uploadedSize / totalSize)}`),
    uploadFileChunkBuffer: async (chainBufferPacket, {chunkIndex}) => withRetryAsync(async () => authFetch(urlFileUpload, {
      method: "POST",
      body: chainBufferPacket,
      timeout
    }).catch(error => {
      const message = `[ERROR][Upload] upload chunk ${chunkIndex} of ${key}`;
      log && log(message, error);
      throw new Error(message);
    }), maxRetry, wait)
  });
  log && log(`[Upload] done: ${key}`);
};
const fileDownload = async ({fileOutputPath, key, urlFileDownload, timeout = 3e4, authFetch, logger, log = logger && logger.add}) => {
  log && log(`[Download] key: ${key}`);
  const fileBuffer = await (await authFetch(`${urlFileDownload}/${encodeURIComponent(key)}`, {
    method: "GET",
    timeout
  })).buffer();
  log && log(`[Download] get: ${binary(fileBuffer.length)}B`);
  if (fileOutputPath) {
    await createDirectory(dirname(fileOutputPath));
    await writeFileAsync(fileOutputPath, fileBuffer);
    log && log(`[Download] done: ${fileOutputPath}`);
  }
  return fileBuffer;
};
const pathAction = async ({nameList = [ "" ], actionType, key = "", keyTo, urlPathAction, timeout = 3e4, authFetch, logger, log = logger && logger.add}) => {
  log && log(`[Action|${actionType}] key: ${key}, keyTo: ${keyTo}, nameList: [${nameList}]`);
  const result = await (await authFetch(urlPathAction, {
    method: "POST",
    body: JSON.stringify({
      nameList,
      actionType,
      key,
      keyTo
    }),
    timeout
  })).json();
  log && log(`[Action|${actionType}] done`);
  return result;
};
export { fileUpload, fileDownload, pathAction };