import { relative } from "path";
import { catchAsync } from "dr-js/module/common/error";
import { visibleAsync, statAsync, createPathPrefixLock, toPosixPath } from "dr-js/module/node/file/function";
import { createDirectory } from "dr-js/module/node/file/File";
import { getDirectorySubInfoList, getDirectoryInfoTree, walkDirectoryInfoTree } from "dr-js/module/node/file/Directory";
import { modify } from "dr-js/module/node/file/Modify";
const PATH_ACTION_TYPE = {
  PATH_VISIBLE: "path:visible",
  PATH_STAT: "path:stat",
  PATH_COPY: "path:copy",
  PATH_MOVE: "path:move",
  PATH_DELETE: "path:delete",
  DIRECTORY_CREATE: "directory:create",
  DIRECTORY_CONTENT: "directory:content",
  DIRECTORY_ALL_FILE_LIST: "directory:all-file-list"
};
const PATH_ACTION_MAP = {
  "path:visible": absolutePath => visibleAsync(absolutePath).then(isVisible => ({
    isVisible
  })),
  "path:stat": absolutePath => statAsync(absolutePath).then(({mode, size, mtimeMs}) => ({
    mode,
    size,
    mtimeMs
  })),
  "path:copy": modify.copy,
  "path:move": modify.move,
  "path:delete": modify.delete,
  "directory:create": absolutePath => createDirectory(absolutePath),
  "directory:content": async absolutePath => {
    const {result: subInfoList, error} = await catchAsync(getDirectorySubInfoList, absolutePath);
    const directoryList = [];
    const fileList = [];
    subInfoList && subInfoList.forEach(({name, stat}) => stat.isDirectory() ? directoryList.push(name) : fileList.push([ name, stat.size, Math.round(stat.mtimeMs) ]));
    return {
      directoryList,
      fileList
    };
  },
  "directory:all-file-list": async absolutePath => {
    const fileList = [];
    const {error} = await catchAsync(async () => walkDirectoryInfoTree(await getDirectoryInfoTree(absolutePath), ({path, stat}) => !stat.isDirectory() && fileList.push([ toPosixPath(relative(absolutePath, path)), stat.size, Math.round(stat.mtimeMs) ])));
    return {
      fileList
    };
  }
};
const createGetPathAction = rootPath => {
  const getPath = createPathPrefixLock(rootPath);
  return async (actionType, key, keyTo) => {
    const absolutePath = getPath(key);
    const absolutePathTo = keyTo && getPath(keyTo);
    return PATH_ACTION_MAP[actionType](absolutePath, absolutePathTo);
  };
};
export { PATH_ACTION_TYPE, createGetPathAction };