import { clock, getTimestamp, createTimer } from "dr-js/module/common/time";
import { withRetryAsync, lossyAsync } from "dr-js/module/common/function";
import { roundFloat } from "dr-js/module/common/math/base";
import { createFactDatabase, tryDeleteExtraCache } from "dr-js/module/node/module/FactDatabase";
import { addExitListenerAsync, addExitListenerSync } from "dr-js/module/node/system/ExitListener";
import { applyStatusFact } from "./module/applyStatusFact";
const configureStatusCollector = async ({collectPath: pathFactDirectory, collectUrl, collectInterval = 3e5, authFetch}) => {
  const factDatabase = await createFactDatabase({
    pathFactDirectory,
    applyFact: applyStatusFact,
    onError: error => console.error("[collectStatus][FactDatabase]", error)
  });
  addExitListenerSync(factDatabase.end);
  addExitListenerAsync(async () => {
    factDatabase.end();
    await factDatabase.getSaveFactCachePromise();
  });
  await tryDeleteExtraCache({
    pathFactDirectory
  });
  const collectStatus = lossyAsync(async () => {
    await withRetryAsync(async retryCount => {
      const timeFetchStart = clock();
      const {json} = await authFetch(collectUrl);
      const timeOk = roundFloat(clock() - timeFetchStart);
      const status = await json();
      const timeDownload = roundFloat(clock() - timeFetchStart - timeOk);
      factDatabase.add({
        timestamp: getTimestamp(),
        retryCount,
        status,
        timeOk,
        timeDownload
      });
    }, 3, 500);
  }, error => {
    console.error("[collectStatus] fetch failed", collectUrl, error);
    factDatabase.add({
      timestamp: getTimestamp(),
      error: error.toString() || "fetch error"
    });
  }).trigger;
  const timer = createTimer({
    func: collectStatus,
    delay: collectInterval
  });
  return {
    factDatabase,
    timer,
    collectStatus
  };
};
export { configureStatusCollector };