const taskListStyle = "<style>\nh2, h6 { margin: 0.5em 4px; }\n.item { display: flex; flex-flow: row nowrap; align-items: stretch; }\n.item:hover { background: #eee; }\n.name { overflow:hidden; flex: 1; white-space:nowrap; text-align: left; text-overflow: ellipsis; background: transparent; }\n.item .name { pointer-events: none; color: #666; }\n.edit { pointer-events: auto; min-width: 1.5em; min-height: auto; line-height: normal; }\n</style>";
const initTaskList = (URL_PATH_ACTION, TASK_ACTION_TYPE, authFetch, withAlertModal, withConfirmModal, withPromptModal, withPromptExtModal) => {
  const {cE, aCL, Dr: {Common: {Compare: {compareString, compareStringWithNumber}, Math: {getRandomId}}}} = window;
  const COMMAND = (a, b) => compareStringWithNumber(a.task.command, b.task.command) || ((a, b) => compareString(a.key, b.key))(a, b);
  const SORT_FUNC = {
    COMMAND,
    NOTE: (a, b) => compareStringWithNumber(a.info.note, b.info.note) || COMMAND(a, b),
    TIME_CREATE: (a, b) => b.info.timeCreate - a.info.timeCreate || COMMAND(a, b),
    TIME_UPDATE: (a, b) => b.info.timeUpdate - a.info.timeUpdate || COMMAND(a, b)
  };
  const SORT_TYPE_LIST = Object.keys(SORT_FUNC);
  const initialTaskListState = {
    taskSortType: SORT_TYPE_LIST[0],
    configList: []
  };
  const authFetchTaskActionJSON = async (type, payload = {}) => (await authFetch(URL_PATH_ACTION, {
    method: "POST",
    body: JSON.stringify({
      type,
      payload
    })
  })).json();
  const getLoadTaskListAsync = async taskListStore => {
    const {configList} = await authFetchTaskActionJSON(TASK_ACTION_TYPE.TASK_LIST);
    taskListStore.setState({
      configList
    });
  };
  return {
    initialTaskListState,
    cycleTaskSortType: (taskListStore, taskSortType = taskListStore.getState().taskSortType) => {
      const nextSortIndex = (SORT_TYPE_LIST.indexOf(taskSortType) + 1) % SORT_TYPE_LIST.length;
      taskListStore.setState({
        taskSortType: SORT_TYPE_LIST[nextSortIndex]
      });
    },
    authFetchTaskActionJSON,
    getLoadTaskListAsync,
    getTaskActionAsync: taskListStore => async (type, payload) => {
      await authFetchTaskActionJSON(type, payload);
      await getLoadTaskListAsync(taskListStore);
    },
    getSetTaskConfigAsync: taskAction => async (config = {
      key: getRandomId("Task-"),
      task: {},
      info: {}
    }) => {
      const resultList = await withPromptExtModal([ [ "task.command", config.task.command || "" ], [ "task.argList", JSON.stringify(config.task.argList) || "[]" ], [ "task.cwd", config.task.cwd || "" ], [ "task.env", JSON.stringify(config.task.env) || "{}" ], [ "task.shell", config.task.shell || "true" ], [ "task.resetLog", config.task.resetLog || "true" ], [ "info.note", config.info.note || "" ] ]);
      if (!resultList) return;
      const [taskCommand, taskArgList, taskCwd, taskEnv, taskShell, taskResetLog, infoNote] = resultList;
      return taskAction(TASK_ACTION_TYPE.TASK_CONFIG_SET, {
        key: config.key,
        task: {
          command: taskCommand,
          argList: JSON.parse(taskArgList),
          cwd: taskCwd,
          env: JSON.parse(taskEnv),
          shell: "true" === taskShell,
          resetLog: "true" === taskResetLog
        },
        info: {
          note: infoNote
        }
      });
    },
    renderTaskList: (taskListStore, parentElement, loadTaskList, taskAction, setTaskConfigAsync) => {
      const {taskSortType, configList} = taskListStore.getState();
      parentElement.innerHTML = "";
      aCL(parentElement, [ cE("h2", {
        innerText: "Task List"
      }), cE("h6", {
        innerText: `${configList.length} task`
      }), ...configList.sort(SORT_FUNC[taskSortType]).map(config => {
        const {key, task, info, status} = config;
        const isRunning = Boolean(status && status.processInfo);
        return cE("div", {
          className: "item"
        }, [ cE("span", {
          className: "name button",
          innerText: `📄|${info.note ? `[Note] ${info.note} ` : ""}[Command] ${task.command}`
        }), cE("button", {
          className: "edit",
          innerText: "🔎",
          onclick: async () => withAlertModal(JSON.stringify(config, null, 2))
        }), cE("button", {
          className: "edit",
          innerText: "🔧",
          disabled: isRunning,
          onclick: () => setTaskConfigAsync(config)
        }), cE("button", {
          className: "edit",
          innerText: isRunning ? "⏹" : "▶️",
          onclick: async () => await withConfirmModal(`${isRunning ? "Stop" : "Start"} task: ${key}?`) && taskAction(isRunning ? TASK_ACTION_TYPE.TASK_STOP : TASK_ACTION_TYPE.TASK_START, config)
        }), cE("button", {
          className: "edit",
          innerText: "📃",
          onclick: async () => withAlertModal(await (async (type, payload = {}) => (await authFetch(URL_PATH_ACTION, {
            method: "POST",
            body: JSON.stringify({
              type,
              payload
            })
          })).text())(await withConfirmModal(`Get tail instead of full log for task: ${config.key}?`, "Tail log", "Full log") ? TASK_ACTION_TYPE.TASK_LOG_GET_TAIL : TASK_ACTION_TYPE.TASK_LOG_GET, config) || "[no log]")
        }), cE("button", {
          className: "edit",
          innerText: "♻️",
          onclick: async () => await withConfirmModal(`Reset log of task: ${key}?`) && taskAction(TASK_ACTION_TYPE.TASK_LOG_RESET, config)
        }), cE("button", {
          className: "edit",
          innerText: "🗑️",
          disabled: isRunning,
          onclick: async () => await withConfirmModal(`Delete task: ${key}?`) && taskAction(TASK_ACTION_TYPE.TASK_DELETE, config)
        }) ]);
      }) ]);
    }
  };
};
export { taskListStyle, initTaskList };