import { dirname } from "path";
import { writeFileSync } from "fs";
import { catchAsync, catchSync } from "dr-js/module/common/error";
import { getTimestamp } from "dr-js/module/common/time";
import { createCacheMap } from "dr-js/module/common/data/CacheMap";
import { readFileAsync } from "dr-js/module/node/file/function";
import { createDirectory } from "dr-js/module/node/file/File";
import { addExitListenerSync } from "dr-js/module/node/system/ExitListener";
import { getRandomBufferAsync } from "dr-js/module/node/data/function";
const DEFAULT_TOKEN_KEY = "auth-token";
const loadTokenCache = async (tokenCacheMap, fileTokenCache) => {
  tokenCacheMap.loadCacheList(JSON.parse(await readFileAsync(fileTokenCache)));
};
const saveTokenCache = (tokenCacheMap, fileTokenCache) => {
  const tokenPackString = JSON.stringify(tokenCacheMap.saveCacheList());
  writeFileSync(fileTokenCache, tokenPackString);
};
const configureTokenCache = async ({fileTokenCache, tokenKey = "auth-token", tokenSize = 128, tokenExpireTime = 2592e6, tokenSizeSumMax = 4096, tokenCacheMap = createCacheMap({
  valueSizeSumMax: tokenSizeSumMax,
  valueSizeSingleMax: 1,
  eventHub: null
})}) => {
  await createDirectory(dirname(fileTokenCache));
  await catchAsync(loadTokenCache, tokenCacheMap, fileTokenCache);
  addExitListenerSync(exitState => {
    catchSync(saveTokenCache, tokenCacheMap, fileTokenCache);
  });
  return {
    tokenKey,
    tokenExpireTime,
    tryGetToken: token => tokenCacheMap.get(token),
    generateToken: async tokenObject => {
      const timestamp = getTimestamp();
      const token = `${(await getRandomBufferAsync(tokenSize)).toString("base64").replace(/\W/g, "")}-${timestamp.toString(36)}`;
      tokenCacheMap.set(token, {
        ...tokenObject,
        timestamp
      }, 1, Date.now() + tokenExpireTime);
      return token;
    }
  };
};
export { DEFAULT_TOKEN_KEY, configureTokenCache };