import { binary } from "dr-js/module/common/format";
import { parseCookieString } from "dr-js/module/node/server/function";
import { runQuiet } from "dr-js/module/node/system/Run";
import { describeSystemStatus } from "dr-js/module/node/system/Status";
const getParamFromRequest = (store, key) => {
  const {headers} = store.request;
  return headers[key] || store.getState().url.searchParams.get(key) || headers.cookie && decodeURIComponent(parseCookieString(headers.cookie)[key]);
};
const isPrivateAddress = address => "127.0.0.1" === address || "::1" === address || "::" === address || address.startsWith("192.168.") || address.startsWith("127.") || address.startsWith("10.") || address.startsWith("fd") || "localhost" === address;
const COMMON_SERVER_STATUS_COMMAND_LIST = [ [ "Path", [ "du -hd1" ] ], [ "Disk", [ "df -h .", {
  command: 'dir | find "bytes free"',
  processOutput: stdout => `${binary(Number(stdout.match(/([\d,]+) bytes free/)[1].replace(/\D/g, "")))}B storage free`
} ] ], [ "Network", [ "vnstat -s" ] ], [ "System", [ "top -b -n 1 | head -n 5", {
  commandFunc: () => describeSystemStatus()
} ] ], [ "Time", [ {
  commandFunc: () => new Date().toISOString()
} ] ] ];
const tryGetStatus = async (option, rootPath) => {
  "string" == typeof option && (option = {
    command: option
  });
  let output = null;
  option.command && (output = await (async (command, rootPath) => {
    const {promise, stdoutBufferPromise} = runQuiet({
      command,
      option: {
        cwd: rootPath
      }
    });
    await promise;
    return (await stdoutBufferPromise).toString();
  })(option.command, rootPath));
  option.commandFunc && (output = await option.commandFunc(option));
  option.processOutput && (output = option.processOutput(output));
  return output;
};
const getCommonServerStatus = async rootPath => {
  const resultList = [];
  for (const [title, tryList] of COMMON_SERVER_STATUS_COMMAND_LIST) {
    let output = "";
    for (const option of tryList) if (output = await tryGetStatus(option, rootPath).catch(() => "")) break;
    resultList.push([ title, output ]);
  }
  return resultList;
};
export { getParamFromRequest, isPrivateAddress, getCommonServerStatus };