var _Server = require("dr-js/library/node/server/Server");
var _Common = require("dr-js/library/node/server/Responder/Common");
var _Send = require("dr-js/library/node/server/Responder/Send");
var _Router = require("dr-js/library/node/server/Responder/Router");
var _log = require("../library/configure/log");
var _pid = require("../library/configure/pid");
var _permission = require("../library/configure/permission");
var _server = require("../library/configure/server");
var _Common2 = require("../library/responder/Common");
var _configureFeaturePack = require("../library/feature/Auth/configureFeaturePack");
var _configureFeaturePack2 = require("../library/feature/Explorer/configureFeaturePack");
var _configureFeaturePack3 = require("../library/feature/StatusCollect/configureFeaturePack");
var _configureFeaturePack4 = require("../library/feature/StatusReport/configureFeaturePack");
var _configureFeaturePack5 = require("../library/feature/TaskRunner/configureFeaturePack");
exports.createServer = async ({protocol, hostname, port, fileSSLKey, fileSSLCert, fileSSLChain, fileSSLDHParam, pathLogDirectory, logFilePrefix, filePid, shouldIgnoreExistPid, permissionType, permissionFunc, permissionFile, authSkip = !1, authFile, authFileGenTag, authFileGenSize, authFileGenTokenSize, authFileGenTimeGap, authFileGroupPath, authFileGroupDefaultTag, authFileGroupKeySuffix, explorerRootPath, explorerUploadMergePath, statusCollectPath, statusCollectUrl, statusCollectInterval, statusReportProcessTag, taskRunnerRootPath}) => {
  await (0, _pid.configurePid)({
    filePid,
    shouldIgnoreExistPid
  });
  const {server, start, stop, option} = await (0, _server.configureServer)({
    protocol,
    hostname,
    port,
    fileSSLKey,
    fileSSLCert,
    fileSSLChain,
    fileSSLDHParam
  });
  const logger = await (0, _log.configureLog)({
    pathLogDirectory,
    logFilePrefix
  });
  const {checkPermission} = await (0, _permission.configurePermission)({
    permissionType,
    permissionFunc,
    permissionFile,
    logger
  });
  const responderLogEnd = (0, _Common.createResponderLogEnd)({
    log: logger.add
  });
  const featureAuth = await (0, _configureFeaturePack.configureFeaturePack)(Object.assign({}, {
    option,
    logger,
    routePrefix: ""
  }, {}, {
    authSkip
  }, {}, {
    authFile,
    authFileGenTag,
    authFileGenSize,
    authFileGenTokenSize,
    authFileGenTimeGap
  }, {}, {
    authFileGroupPath,
    authFileGroupDefaultTag,
    authFileGroupKeySuffix
  }, {
    URL_AUTH_CHECK: "/auth"
  }));
  const featureExplorer = explorerRootPath && await (0, _configureFeaturePack2.configureFeaturePack)(Object.assign({}, {
    option,
    logger,
    routePrefix: "",
    featureAuth
  }, {
    explorerRootPath,
    explorerUploadMergePath,
    checkPermission
  }));
  const featureStatusCollect = statusCollectPath && await (0, _configureFeaturePack3.configureFeaturePack)(Object.assign({}, {
    option,
    logger,
    routePrefix: "",
    featureAuth
  }, {
    statusCollectPath,
    statusCollectUrl,
    statusCollectInterval
  }));
  const featureStatusReport = statusReportProcessTag && await (0, _configureFeaturePack4.configureFeaturePack)(Object.assign({}, {
    option,
    logger,
    routePrefix: "",
    featureAuth
  }, {
    statusReportProcessTag
  }));
  const featureTaskRunner = taskRunnerRootPath && await (0, _configureFeaturePack5.configureFeaturePack)(Object.assign({}, {
    option,
    logger,
    routePrefix: "",
    featureAuth
  }, {
    taskRunnerRootPath,
    checkPermission
  }));
  const redirectUrl = featureExplorer ? featureExplorer.URL_HTML : featureStatusCollect ? featureStatusCollect.URL_HTML : featureStatusReport ? featureStatusReport.URL_HTML : featureTaskRunner ? featureTaskRunner.URL_HTML : "";
  const routeMap = (0, _Router.createRouteMap)([ [ [ "/favicon", "/favicon.ico" ], "GET", (0, _Send.createResponderFavicon)() ], [ "/", "GET", redirectUrl ? store => (0, _Common.responderEndWithRedirect)(store, {
    redirectUrl
  }) : store => (0, _Common.responderEndWithStatusCode)(store, {
    statusCode: 400
  }) ], ...featureAuth.routeList, ...featureExplorer ? featureExplorer.routeList : [], ...featureStatusCollect ? featureStatusCollect.routeList : [], ...featureStatusReport ? featureStatusReport.routeList : [], ...featureTaskRunner ? featureTaskRunner.routeList : [] ].filter(Boolean));
  server.on("request", (0, _Server.createRequestListener)({
    responderList: [ (0, _Common.createResponderLog)({
      log: logger.add
    }), _Common2.responderCommonExtend, (0, _Router.createResponderRouter)({
      routeMap,
      baseUrl: option.baseUrl
    }) ],
    responderEnd: store => {
      (0, _Common.responderEnd)(store);
      responderLogEnd(store);
    }
  }));
  return {
    server,
    start,
    stop,
    option,
    logger,
    featureAuth,
    featureExplorer,
    featureStatusCollect,
    featureStatusReport,
    featureTaskRunner
  };
};